# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import time

from driverng.protocol.mllp import MLLPProtocol
from driverng.utils import FS, VT


class Cobas6800Protocol(MLLPProtocol):
    HL7_ACK = b"""MSH|^~\\&|%(sender_id)s||%(receiver_id)s||%(date)s||%(message_type)s|%(control_id)s|P|2.5\rMSA|AA|%(ack_id)s|\r"""
    HL7_QACK = b"""MSH|^~\\&|%(sender_id)s||%(receiver_id)s||%(date)s||%(message_type)s|%(control_id)s||2.5||||||ASCII\rMSA|AA|%(ack_id)s|\rQAK||AA\rQPD|WOS^Work Order Step^IHE_LABTF|%(qpd_sample_id)s||%(qpd_rack)s|%(qpd_position)s"""

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted_payload = payload.splitlines()
        splitted = payload.splitlines()[0].split(b"|")
        qpd_sample_id = b""
        qpd_rack = b""
        qpd_position = b""
        for line in splitted_payload:
            splitted_qpd = b""
            if b"QPD" in line:
                splitted_qpd = line.split(b"|")
                if len(splitted_qpd) > 3:
                    qpd_sample_id = splitted_qpd[3]
                if len(splitted_qpd) > 5:
                    qpd_rack = splitted_qpd[5]
                if len(splitted_qpd) > 6:
                    qpd_position = splitted_qpd[6]
        date = date or datetime.datetime.now()
        type_message = splitted[8]
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"control_id": control_id,
            b"sender_id": splitted[4],
            b"receiver_id": splitted[2],
            b"ack_id": splitted[9],
            b"qpd_sample_id": qpd_sample_id,
            b"qpd_rack": qpd_rack,
            b"qpd_position": qpd_position,
            b"message_type": self.ACK_MESSAGE_TYPE[self.is_automate],
        }
        if type_message.startswith(b"QBP"):
            context[b"message_type"] = b"RSP^K11"
            return Cobas6800Protocol.HL7_QACK % context
        else:
            context[b"message_type"] = b"ACK^R22"
            return Cobas6800Protocol.HL7_ACK % context
        return self.HL7_ACK % context

    def recv_trames(self, timeout=None):
        data = self.mllp_readline(timeout)
        messages = []
        acks = []
        # Plusieurs réceptions de query notamment qui sont à splitter tous les séparateurs FS CR
        for d in data.split(b"\x1c\x0d"):
            if d:
                payload = self.get_payload(d)
                self.logger.info("recv trames: %s ", payload)
                ack = self.get_ack_message(payload)
                if ack:
                    acks.append(ack)
                messages.append(payload)
        for ack in acks:
            self.write(self.build_trames(ack))
        return messages

    def build_trames(self, message):
        # plusieurs messages MSH possibles
        # <vt><payload><fs><cr>
        msg = b""
        for m in message.split(b"MSH|"):
            if m:
                msg += b"%c%s\r%c\r" % (VT, b"\r".join((b"MSH|" + m).splitlines()), FS)
        return msg
