# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "bactalert"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||LIS|||||BACT/ALERT||P|1|20130715141912
    P|1|036666666|||NYME^ANO||20110513|F
    O|1|123456||^^^A|||20130513114242||||N||||Serum
    O|2|123456||^^^B|||20130513114242||||A||||Serum
    O|3|123456||^^^C|||20130513114242||||C||||Serum
    L|1|F
    """
    assert message == expected


def test_encode_order_option_virtuo(self):
    self.configure(bact_version_protocole=2)
    message = self.order()
    expected = """
    H|\\^&|||LIS|||||BACT/ALERT||P|1|20130715141912
    P|1|036666666|||NYME^ANO
    O|1|123456||^^^A|||20130513114242||||N||||Serum
    O|2|123456||^^^B|||20130513114242||||A||||Serum
    O|3|123456||^^^C|||20130513114242||||C||||Serum
    L|1|F
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||LIS|||||BACT/ALERT||P|1|20170516164700
    Q|1|ALL||||||||||N
    L|1|F
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "ALL"}],
    }
    assert worklist == expected


def test_decode_result_option_virtuo(self):
    self.configure(bact_version_protocole=2)
    worklist = self.decode(
        """
    H|\\^&|||SENDERNAME|||||BACT/ALERT||P|1|20100128164942
    P|1|G406270/A|||||||||||||||||||||||
    O|1|STSC5001|||||20090302105501||||||||blood^Left Arm
    R|1|^^^BC^BSA^SA023023^7
    R|2|^^^BC^BSN^SN021883^10
    L|1|F
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "STSC5001",
                "results": [
                    {"code": "BC_BSA", "date": None, "value": ""},
                    {"code": "BOTTLE_TYPE_BSA", "date": None, "value": "BSA"},
                    {"code": "BOTTLE_ID_BSA", "date": None, "value": "SA023023"},
                    {"code": "BC_BSN", "date": None, "value": ""},
                    {"code": "BOTTLE_TYPE_BSN", "date": None, "value": "BSN"},
                    {"code": "BOTTLE_ID_BSN", "date": None, "value": "SN021883"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_option_virtuo_with_reagent(self):
    date = datetime.datetime(2019, 5, 28, 10, 19, 25)
    self.configure(bact_version_protocole=2)
    worklist = self.decode(
        """
    H|\\^&|||BACT/ALERT^A.00|||||||P|1|20190528160031
    P|1|9500000755
    O|1|95541204||||||||||||||||||||20190528151912|||F
    R|1|^^^BC^BSA^SA333333|Neg|||H||F|||20190521102432|20190528101925|A2
    R|2|^^^SVM^BSA^SA333333|15|mL||H||F|||20190521102432|20190528101925|A2
    M|1|SA333333|^BOTTLE^BSA^^^20190521^102432^20190528^102012|A2
    L|1|F
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "95541204",
                "results": [
                    {"code": "BC_BSA", "date": date, "value": "Neg"},
                    {"code": "BOTTLE_TYPE_BSA", "date": date, "value": "BSA"},
                    {"code": "BOTTLE_ID_BSA", "date": date, "value": "SA333333"},
                    {"code": "POS_BOTTLE_BSA", "date": date, "value": "A2"},
                    {"code": "INTRODUCE_DATETIME_BSA", "value": "2019/05/21 10:24:32"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BSA",
                        "value": "20190521102432",
                    },
                    {"code": "INTRODUCE_DATE_BSA", "value": "2019/05/21"},
                    {"code": "INTRODUCE_TIME_BSA", "value": "10:24:32"},
                    {"code": "INTRODUCTION_DATE_BSA", "value": "21/05/2019"},
                    {"code": "INTRODUCTION_HEURE_BSA", "value": "10:24"},
                    {"code": "DETERMINED_DATETIME_BSA", "value": "2019/05/28 10:19:25"},
                    {"code": "TIME_TO_DETECTION_BSA", "value": "167.9"},
                    {
                        "code": "SVM_BSA",
                        "date": date,
                        "lots": [
                            {
                                "expiration": datetime.datetime(2019, 5, 21, 0, 0),
                                "lot": "102432",
                                "type": "reagent",
                            }
                        ],
                        "value": "15",
                    },
                    {"code": "BOTTLE_TYPE_BSA", "date": date, "value": "BSA"},
                    {"code": "BOTTLE_ID_BSA", "date": date, "value": "SA333333"},
                    {"code": "POS_BOTTLE_BSA", "date": date, "value": "A2"},
                    {"code": "INTRODUCE_DATETIME_BSA", "value": "2019/05/21 10:24:32"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BSA",
                        "value": "20190521102432",
                    },
                    {"code": "INTRODUCE_DATE_BSA", "value": "2019/05/21"},
                    {"code": "INTRODUCE_TIME_BSA", "value": "10:24:32"},
                    {"code": "INTRODUCTION_DATE_BSA", "value": "21/05/2019"},
                    {"code": "INTRODUCTION_HEURE_BSA", "value": "10:24"},
                    {"code": "DETERMINED_DATETIME_BSA", "value": "2019/05/28 10:19:25"},
                    {"code": "TIME_TO_DETECTION_BSA", "value": "167.9"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2017, 12, 23, 10, 31, 49)
    worklist = self.decode(
        """
    H|\\^&|||BACT/ALERT^A.00|||||BACT/ALERT||P|1|20170614152250
    P|1|
    O|1|611206711001|611206711001||||||||||||||||||||||F
    R|1|^^^BC^BFN+^NRHG0HYM|-|||||F|||20170614102747|20171223103149|1B14
    R|2|^^^BC^BFA+^ARNCDW5B|-|||||P|||20170614102742|20171223103149|1B13
    R|3|^^^BC^BFA+^ARNCDW5B|-|||||F|||20170614102742|20171223103149|1B13
    R|4|^^^SVM^BFN+^SNJN2W28|1|mL||6||F|||20170614102742|20171223103149|1B13
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "611206711001",
                "results": [
                    {"code": "BC_BFN+", "date": date, "value": "-"},
                    {"code": "BOTTLE_TYPE_BFN+", "date": date, "value": "BFN+"},
                    {"code": "BOTTLE_ID_BFN+", "date": date, "value": "NRHG0HYM"},
                    {"code": "POS_BOTTLE_BFN+", "date": date, "value": "1B14"},
                    {"code": "INTRODUCE_DATETIME_BFN+", "value": "2017/06/14 10:27:47"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFN+",
                        "value": "20170614102747",
                    },
                    {"code": "INTRODUCE_DATE_BFN+", "value": "2017/06/14"},
                    {"code": "INTRODUCE_TIME_BFN+", "value": "10:27:47"},
                    {"code": "INTRODUCTION_DATE_BFN+", "value": "14/06/2017"},
                    {"code": "INTRODUCTION_HEURE_BFN+", "value": "10:27"},
                    {
                        "code": "DETERMINED_DATETIME_BFN+",
                        "value": "2017/12/23 10:31:49",
                    },
                    {"code": "TIME_TO_DETECTION_BFN+", "value": "4608.1"},
                    {"code": "BC_BFA+", "date": date, "value": "-"},
                    {"code": "BOTTLE_TYPE_BFA+", "date": date, "value": "BFA+"},
                    {"code": "BOTTLE_ID_BFA+", "date": date, "value": "ARNCDW5B"},
                    {"code": "POS_BOTTLE_BFA+", "date": date, "value": "1B13"},
                    {"code": "INTRODUCE_DATETIME_BFA+", "value": "2017/06/14 10:27:42"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFA+",
                        "value": "20170614102742",
                    },
                    {"code": "INTRODUCE_DATE_BFA+", "value": "2017/06/14"},
                    {"code": "INTRODUCE_TIME_BFA+", "value": "10:27:42"},
                    {"code": "INTRODUCTION_DATE_BFA+", "value": "14/06/2017"},
                    {"code": "INTRODUCTION_HEURE_BFA+", "value": "10:27"},
                    {
                        "code": "DETERMINED_DATETIME_BFA+",
                        "value": "2017/12/23 10:31:49",
                    },
                    {"code": "TIME_TO_DETECTION_BFA+", "value": "4608.1"},
                    {
                        "code": "SVM_BFN+",
                        "value": "1",
                        "date": datetime.datetime(2017, 12, 23, 10, 31, 49),
                    },
                    {
                        "code": "BOTTLE_TYPE_BFN+",
                        "value": "BFN+",
                        "date": datetime.datetime(2017, 12, 23, 10, 31, 49),
                    },
                    {
                        "code": "BOTTLE_ID_BFN+",
                        "value": "SNJN2W28",
                        "date": datetime.datetime(2017, 12, 23, 10, 31, 49),
                    },
                    {
                        "code": "POS_BOTTLE_BFN+",
                        "value": "1B13",
                        "date": datetime.datetime(2017, 12, 23, 10, 31, 49),
                    },
                    {"code": "INTRODUCE_DATETIME_BFN+", "value": "2017/06/14 10:27:42"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFN+",
                        "value": "20170614102742",
                    },
                    {"code": "INTRODUCE_DATE_BFN+", "value": "2017/06/14"},
                    {"code": "INTRODUCE_TIME_BFN+", "value": "10:27:42"},
                    {"code": "INTRODUCTION_DATE_BFN+", "value": "14/06/2017"},
                    {"code": "INTRODUCTION_HEURE_BFN+", "value": "10:27"},
                    {
                        "code": "DETERMINED_DATETIME_BFN+",
                        "value": "2017/12/23 10:31:49",
                    },
                    {"code": "TIME_TO_DETECTION_BFN+", "value": "4608.1"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_integrate_all(self):
    self.configure(bact_inclure_intermediaire=True)
    date = datetime.datetime(2019, 1, 3, 9, 54, 15)
    worklist = self.decode(
        """
    H|\\^&|||BACT/ALERT^A.00|||||BACT/ALERT||P|1|20190103095417
    P|1|177566
    O|1|998489223028|998489223028||||||||||||||||||||||I
    R|1|^^^BC^BFA+^ARS06QPH|*|||||I|||20190103095415|20190103095415|1A51
    R|2|^^^TTD^BFA+^ARS06QPH|0000.0|||||I|||20190103095415|20190103095415|1A51
    L|1|F
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "998489223028",
                "results": [
                    {"code": "BC_BFA+", "date": date, "value": "*"},
                    {"code": "BOTTLE_TYPE_BFA+", "date": date, "value": "BFA+"},
                    {"code": "BOTTLE_ID_BFA+", "date": date, "value": "ARS06QPH"},
                    {"code": "POS_BOTTLE_BFA+", "date": date, "value": "1A51"},
                    {"code": "INTRODUCE_DATETIME_BFA+", "value": "2019/01/03 09:54:15"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFA+",
                        "value": "20190103095415",
                    },
                    {"code": "INTRODUCE_DATE_BFA+", "value": "2019/01/03"},
                    {"code": "INTRODUCE_TIME_BFA+", "value": "09:54:15"},
                    {"code": "INTRODUCTION_DATE_BFA+", "value": "03/01/2019"},
                    {"code": "INTRODUCTION_HEURE_BFA+", "value": "09:54"},
                    {
                        "code": "DETERMINED_DATETIME_BFA+",
                        "value": "2019/01/03 09:54:15",
                    },
                    {"code": "TIME_TO_DETECTION_BFA+", "value": "0.0"},
                    {"code": "TTD_BFA+", "date": date, "value": "0000.0"},
                    {"code": "BOTTLE_TYPE_BFA+", "date": date, "value": "BFA+"},
                    {"code": "BOTTLE_ID_BFA+", "date": date, "value": "ARS06QPH"},
                    {"code": "POS_BOTTLE_BFA+", "date": date, "value": "1A51"},
                    {"code": "INTRODUCE_DATETIME_BFA+", "value": "2019/01/03 09:54:15"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFA+",
                        "value": "20190103095415",
                    },
                    {"code": "INTRODUCE_DATE_BFA+", "value": "2019/01/03"},
                    {"code": "INTRODUCE_TIME_BFA+", "value": "09:54:15"},
                    {"code": "INTRODUCTION_DATE_BFA+", "value": "03/01/2019"},
                    {"code": "INTRODUCTION_HEURE_BFA+", "value": "09:54"},
                    {
                        "code": "DETERMINED_DATETIME_BFA+",
                        "value": "2019/01/03 09:54:15",
                    },
                    {"code": "TIME_TO_DETECTION_BFA+", "value": "0.0"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_intermediaire(self):
    self.configure(bact_inclure_intermediaire=True)
    date = datetime.datetime(2020, 1, 31, 20, 7, 6)
    worklist = self.decode(
        """
    H|\\^&|||BACT/ALERT^A.00|||||BACT/ALERT||P|1|20200131200808
    P|1|147855
    O|1|2401AK00128287|2401AK00128287||||||||||||||||||||||I
    R|1|^^^BC^BFA+^ART15T02|*|||||I|||20200128200213|20200131200706|1B53
    R|2|^^^TTD^BFA+^ART15T02|0072.0|||||I|||20200128200213|20200131200706|1B53
    L|1|F
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "2401AK00128287",
                "results": [
                    {"code": "BC_BFA+", "date": date, "value": "*"},
                    {"code": "BOTTLE_TYPE_BFA+", "date": date, "value": "BFA+"},
                    {"code": "BOTTLE_ID_BFA+", "date": date, "value": "ART15T02"},
                    {"code": "POS_BOTTLE_BFA+", "date": date, "value": "1B53"},
                    {"code": "INTRODUCE_DATETIME_BFA+", "value": "2020/01/28 20:02:13"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFA+",
                        "value": "20200128200213",
                    },
                    {"code": "INTRODUCE_DATE_BFA+", "value": "2020/01/28"},
                    {"code": "INTRODUCE_TIME_BFA+", "value": "20:02:13"},
                    {"code": "INTRODUCTION_DATE_BFA+", "value": "28/01/2020"},
                    {"code": "INTRODUCTION_HEURE_BFA+", "value": "20:02"},
                    {
                        "code": "DETERMINED_DATETIME_BFA+",
                        "value": "2020/01/31 20:07:06",
                    },
                    {"code": "TIME_TO_DETECTION_BFA+", "value": "72.1"},
                    {"code": "TTD_BFA+", "date": date, "value": "0072.0"},
                    {"code": "BOTTLE_TYPE_BFA+", "date": date, "value": "BFA+"},
                    {"code": "BOTTLE_ID_BFA+", "date": date, "value": "ART15T02"},
                    {"code": "POS_BOTTLE_BFA+", "date": date, "value": "1B53"},
                    {"code": "INTRODUCE_DATETIME_BFA+", "value": "2020/01/28 20:02:13"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFA+",
                        "value": "20200128200213",
                    },
                    {"code": "INTRODUCE_DATE_BFA+", "value": "2020/01/28"},
                    {"code": "INTRODUCE_TIME_BFA+", "value": "20:02:13"},
                    {"code": "INTRODUCTION_DATE_BFA+", "value": "28/01/2020"},
                    {"code": "INTRODUCTION_HEURE_BFA+", "value": "20:02"},
                    {
                        "code": "DETERMINED_DATETIME_BFA+",
                        "value": "2020/01/31 20:07:06",
                    },
                    {"code": "TIME_TO_DETECTION_BFA+", "value": "72.1"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_ignore(self):
    self.configure(bact_inclure_preliminaire=True, bact_format_delay_positivity=2)
    date = datetime.datetime(2018, 5, 26, 7, 56, 27)
    worklist = self.decode(
        """
    H|\\^&|||BACT/ALERT^A.00|||||BACT/ALERT||P|1|20170614152250
    P|1|92359
    O|1|788052513055|788052513055||||||||||||||||||||||P
    R|1|^^^BC^BFA+^ARR78Y94|+|||||P|||20180525140339|20180526075627|1D33
    R|2|^^^TTD^BFA+^ARR78Y94|0017.8|||||P|||20180525140339|20180526075627|1D33
    R|3|^^^BC^BFN+^NRJHYQR2|*|||||I|||20180525140334|20180526075627|1D23
    R|4|^^^TTD^BFN+^NRJHYQR2|0017.8|||||I|||20180525140334|20180526075627|1D23
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "788052513055",
                "results": [
                    {"code": "BC_BFA+", "date": date, "value": "+"},
                    {"code": "BOTTLE_TYPE_BFA+", "date": date, "value": "BFA+"},
                    {"code": "BOTTLE_ID_BFA+", "date": date, "value": "ARR78Y94"},
                    {"code": "POS_BOTTLE_BFA+", "date": date, "value": "1D33"},
                    {"code": "INTRODUCE_DATETIME_BFA+", "value": "2018/05/25 14:03:39"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFA+",
                        "value": "20180525140339",
                    },
                    {"code": "INTRODUCE_DATE_BFA+", "value": "2018/05/25"},
                    {"code": "INTRODUCE_TIME_BFA+", "value": "14:03:39"},
                    {"code": "INTRODUCTION_DATE_BFA+", "value": "25/05/2018"},
                    {"code": "INTRODUCTION_HEURE_BFA+", "value": "14:03"},
                    {
                        "code": "DETERMINED_DATETIME_BFA+",
                        "value": "2018/05/26 07:56:27",
                    },
                    {
                        "code": "POSITIVITY_DATETIME_BFA+",
                        "value": "2018/05/26 07:56:27",
                    },
                    {
                        "code": "BOTTLE_POSITIVITY_DATETIME_BFA+",
                        "value": "20180526075627",
                    },
                    {"code": "POSITIVITY_DATE_BFA+", "value": "2018/05/26"},
                    {"code": "POSITIVITY_TIME_BFA+", "value": "07:56:27"},
                    {"code": "POSITIVITE_DATE_BFA+", "value": "26/05/2018"},
                    {"code": "POSITIVITE_HEURE_BFA+", "value": "07:56"},
                    {"code": "TIME_TO_DETECTION_POSITIVITY_BFA+", "value": "17:52"},
                    {"code": "TIME_TO_DETECTION_BFA+", "value": "17.9"},
                    {"code": "TTD_BFA+", "date": date, "value": "0017.8"},
                    {"code": "BOTTLE_TYPE_BFA+", "date": date, "value": "BFA+"},
                    {"code": "BOTTLE_ID_BFA+", "date": date, "value": "ARR78Y94"},
                    {"code": "POS_BOTTLE_BFA+", "date": date, "value": "1D33"},
                    {"code": "INTRODUCE_DATETIME_BFA+", "value": "2018/05/25 14:03:39"},
                    {
                        "code": "BOTTLE_INTRODUCE_DATETIME_BFA+",
                        "value": "20180525140339",
                    },
                    {"code": "INTRODUCE_DATE_BFA+", "value": "2018/05/25"},
                    {"code": "INTRODUCE_TIME_BFA+", "value": "14:03:39"},
                    {"code": "INTRODUCTION_DATE_BFA+", "value": "25/05/2018"},
                    {"code": "INTRODUCTION_HEURE_BFA+", "value": "14:03"},
                    {
                        "code": "DETERMINED_DATETIME_BFA+",
                        "value": "2018/05/26 07:56:27",
                    },
                    {"code": "TIME_TO_DETECTION_BFA+", "value": "17.9"},
                ],
            }
        ],
    }
    assert worklist == expected
