# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "helios"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].results.add(
        code="ANCA_DILUTION", value="80;160;320", anterior=False
    )
    message = self.order(worklist=worklist)
    expected = """
    H|\\^&|||||||||||LIS2-A2|20130715141912
    P|1||123456||NYME^ANO
    O|1|123456||^^^A^1:10|R
    O|2|123456||^^^B|R
    O|3|123456||^^^ANCA,80|R
    O|4|123456||^^^ANCA,160|R
    O|5|123456||^^^ANCA,320|R
    L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||Helmed_IFA|||||||P|LIS2-A2|20090128162157
    Q|1|^S001^||ALL
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "S001"}],
    }
    assert worklist == expected


def test_decode_result1(self):
    worklist = self.decode(
        """
    H|\\^&|||Helios|||||||P|1|20171027174554
    P|1||1000000115||^^^^
    O|1|1000000115||^^^ANA-HEp2(IFA)^80|R
    R|1|^^^ANA-Ep2(IFA)^80|negative^^|||||F||user||20170620165133|Helios^0^2
    C|1||IFA^|T
    C|2||IFA :|C
    C|3||ELISA^|T
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "1000000115",
                "rack": "0_2",
                "results": [
                    {
                        "code": "ANA-Ep2(IFA)_RESULT_80",
                        "value": "negative",
                        "comments": ["IFA^^", "ELISA^^"],
                        "date": datetime.datetime(2017, 6, 20, 16, 51, 33),
                        "dilution": "80",
                        "instrument": "Helios",
                        "operator": "user",
                    },
                    {
                        "code": "ANA-Ep2(IFA)_RESULT",
                        "value": "negative",
                        "comments": ["IFA^^", "ELISA^^"],
                        "date": datetime.datetime(2017, 6, 20, 16, 51, 33),
                        "dilution": "80",
                        "instrument": "Helios",
                        "operator": "user",
                    },
                    {
                        "code": "ANA-Ep2(IFA)_TITRE",
                        "value": "80",
                        "date": datetime.datetime(2017, 6, 20, 16, 51, 33),
                        "instrument": "Helios",
                        "operator": "user",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_positive(self):
    worklist = self.decode(
        """
    H|\\^&|||Helios|||||||P|1|20171114164618
    P|1||526090709033||^^^^
    O|1|526090709033||^^^ACNUC^80|R
    R|1|^^^ACNUC^80|1^Centromere^Mitochondrial^negative^1/160|||||F||superuser^superuser^superuser||20171113162000|Helios^2^4^1^13
    C|1|P|ESSAI COMMENTAIRE 1|C
    C|2|P|EEPT^--|T
    C|3|P|EPAT^|T
    C|4|I|http://DELL-PC:8085/bbe41d9779839cd4057d2237593dcba0_pv.jpg^ACAN^80|I
    C|5|I|http://DELL-PC:8085/117bd40492bcab97da24c3a70e9f89bf_pv.jpg^ACAN^80|I
    C|7|I|IFA^IFA-dilutions[1/80;1/160;1/320;1/640]|T
    C|8|I|IFA IFA-dilutions[1/80;1/160;1/320;1/640]:|C
    C|9|I|Unspecified^|T
    C|10|I|Unspecified : Triple Substrat|C
    C|11|I|Unspecified : 160|C
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "526090709033",
                "rack": "2_4",
                "results": [
                    {
                        "code": "ACNUC_COMMENT",
                        "value": "ESSAI COMMENTAIRE 1",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "dilution": "80",
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACAN_1_JPG",
                        "value": "http://DELL-PC:8085/bbe41d9779839cd4057d2237593dcba0_pv.jpg",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                    },
                    {
                        "code": "ACAN_2_JPG",
                        "value": "http://DELL-PC:8085/117bd40492bcab97da24c3a70e9f89bf_pv.jpg",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                    },
                    {
                        "code": "ACNUC_TITRE_MANUEL",
                        "value": "160",
                        "comments": ["Manuel"],
                        "dilution": "80",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_DILUTION",
                        "value": "80;160;320;640",
                        "dilution": "80",
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                    },
                    {
                        "code": "ACNUC_RESULT_80",
                        "value": "1^Centromere^Mitochondrial",
                        "comments": [
                            "EEPT^--^",
                            "EPAT^^",
                            "IFA^IFA-dilutions[1/80;1/160;1/320;1/640]^",
                            "Unspecified^^",
                        ],
                        "dilution": "80",
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                    },
                    {
                        "code": "ACNUC_RESULT",
                        "value": "1^Centromere^Mitochondrial",
                        "comments": [
                            "EEPT^--^",
                            "EPAT^^",
                            "IFA^IFA-dilutions[1/80;1/160;1/320;1/640]^",
                            "Unspecified^^",
                        ],
                        "dilution": "80",
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                    },
                    {
                        "code": "ACNUC_TITRE",
                        "value": "80",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_ASPECT1",
                        "value": "Centromere",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_ASPECT2",
                        "value": "Mitochondrial",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_TITRE2",
                        "value": "80",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_MITOSIS",
                        "value": "negative",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_ENDPTITER",
                        "value": "1/160",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_ASPECT",
                        "value": "Centromere^Mitochondrial",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "operator": "superuser, superuser, superuser",
                        "code": "ACNUC_ASPECT_Centromere",
                        "value": "80",
                        "flags": ["1"],
                        "instrument": "Helios",
                    },
                    {
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "operator": "superuser, superuser, superuser",
                        "code": "ACNUC_ASPECT_Mitochondrial",
                        "value": "80",
                        "flags": ["1"],
                        "instrument": "Helios",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_superieur(self):
    worklist = self.decode(
        """
    H|\\^&|||Helios|||||||P|1|20171114164618
    P|1||526090709033||^^^^
    O|1|526090709033||^^^ACNUC^80|R
    R|1|^^^ACNUC^1280|1^Centromere^^^--|||||F||superuser^superuser^superuser||20171113162000|Helios^2^4^1^13
    C|1|P|EEPT^--|T
    C|2|P|EPAT^|T
    C|4|I|Unspecified : >1280|C
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "526090709033",
                "rack": "2_4",
                "results": [
                    {
                        "code": "ACNUC_RESULT_1280",
                        "value": "1^Centromere",
                        "comments": ["EEPT^--^", "EPAT^^"],
                        "dilution": "1280",
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                    },
                    {
                        "code": "ACNUC_RESULT",
                        "value": "1^Centromere",
                        "comments": ["EEPT^--^", "EPAT^^"],
                        "dilution": "1280",
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                    },
                    {
                        "code": "ACNUC_TITRE",
                        "value": ">1280",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_ASPECT1",
                        "value": "Centromere",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "code": "ACNUC_ASPECT",
                        "value": "Centromere^",
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "instrument": "Helios",
                        "operator": "superuser, superuser, superuser",
                    },
                    {
                        "date": datetime.datetime(2017, 11, 13, 16, 20),
                        "operator": "superuser, superuser, superuser",
                        "code": "ACNUC_ASPECT_Centromere",
                        "value": ">1280",
                        "flags": ["1"],
                        "instrument": "Helios",
                    },
                ],
            }
        ],
    }
    assert worklist == expected
