# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

from driverng import Worklist

parser_class = "immucor"


def test_query(self):
    result = self.decode(
        """H|\\^&|||Arkonet|||||LIS|||LIS2-A2|20050222140243
        Q|1|0651439A||ALL
        L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "0651439A", "type": "patient"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """H|\\^&|||Arkonet|||||LIS|||LIS2-A2|20050222140243
        Q|1|0651439A||ALL
        L|1|N"""
            )
        },
    )
    expected = """H|\\^&|||MCA|||||RVT|||LIS2-A2|20130715141912
        P|1
        O|1|0651439A|||R||||||||||Blood^Patient||||||||||Z
        L|1|N"""
    assert result == expected


def test_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0]["orders"].extend(
        [
            {"code": "ABOD", "action": "add", "priority": "R"},
            {"code": "GrBB", "action": "new", "priority": "R"},
        ]
    )
    result = self.order(worklist=worklist)
    expected = """H|\\^&|||MCA|||||RVT|||LIS2-A2|20130715141912
        P|1|036666666||L036666666|NYME^ANO|NJF|20110513|F|||||||||||||||||LOC|||||||LOCATION
        O|1|123456||^^^A|S||||||N||||Blood^Patient
        O|2|123456||^^^B|S||||||N||||Blood^Patient
        O|3|123456||^^^GrBB|R||||||N||||Blood^Patient
        L|1|N"""
    assert result == expected


def test_result(self):
    date = datetime.datetime(2013, 8, 28, 15, 10, 21)
    result = self.decode(
        """
    H|\\^&|||Arkonet|||||LIS|||LIS2-A2|20130828151021
    P|1|1171984|||Patient^Test||19590422|M
    O|1|0651439A||^^^ABOD Full|R||||||||||Blood^Patient
    R|1|^^^Rh Ctrl|0^0^0|||||F||brentp||20130828151021|M0002
    R|2|^^^Anti-A|0^0^0|||||F||brentp||20130828151021|M0002
    R|3|^^^Anti-B|3+^3+^45|||||F||brentp||20130828151021|M0002
    R|4|^^^Anti-AB|3+^3+^45|||||F||brentp||20130828151021|M0002
    R|5|^^^Anti-D1|3+^3+^29|||||F||brentp||20130828151021|M0002
    R|6|^^^Anti-D2|3+^3+^40|||||F||brentp||20130828151021|M0002
    R|7|^^^A1 Cells|2+^2+^17|||||F||brentp||20130828151021|M0002
    R|8|^^^B Cells|0^0^0|||||F||brentp||20130828151021|M0002
    R|9|^^^ABOD Full Interp|B Pos|||||F||brentp||20130828151021|M0002
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "0651439A",
                "type": "patient",
                "results": [
                    {"code": "NOM", "value": "Patient"},
                    {"code": "PRENOM", "value": "Test"},
                    {"code": "DNAIS", "value": "22/04/1959"},
                    {
                        "date": date,
                        "value": "0",
                        "code": "Rh Ctrl",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "0",
                        "code": "Anti-A",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "3+",
                        "code": "Anti-B",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "3+",
                        "code": "Anti-AB",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "3+",
                        "code": "Anti-D1",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "3+",
                        "code": "Anti-D2",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "2+",
                        "code": "A1 Cells",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "0",
                        "code": "B Cells",
                        "operator": "brentp",
                    },
                    {
                        "date": date,
                        "value": "B Pos",
                        "code": "ABOD Full Interp",
                        "operator": "brentp",
                    },
                ],
            }
        ],
    }
    assert result == expected
