# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "ised"


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||Alcor^iSED^01.00A^01||||||||1|20130301144108
    P|1||12345
    O|1|1234567^A1||^^^ESR|||||||||||||||||||||P
    R|1|^^^ESR^4537-7|10|mm/h||<||F||||20100326172956|99
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "1234567",
                "rack": "A1",
                "results": [
                    {
                        "code": "ESR",
                        "value": "10",
                        "flags": ["<"],
                        "unit": "mm/h",
                        "date": datetime.datetime(2010, 3, 26, 17, 29, 56),
                        "instrument": "99",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(ised_qc_prefix="QC")
    worklist = self.decode(
        """
    H|\\^&|||Alcor^iSED^01.00A^01||||||||1|20130301144108
    P|1||12345
    O|1|QC1^A1||^^^ESR|||||||||||||||||||||P
    R|1|^^^ESR^4537-7|10|mm/h||<||F||||20100326172956|99
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "QC1",
                "sample_id": "QC1",
                "results": [
                    {
                        "code": "ESR",
                        "value": "10",
                        "flags": ["<"],
                        "unit": "mm/h",
                        "date": datetime.datetime(2010, 3, 26, 17, 29, 56),
                        "instrument": "99",
                    }
                ],
            }
        ],
    }
    assert worklist == expected
