# -*- coding: utf8 -*-

from __future__ import unicode_literals

from driverng.tests import BaseTest


class LCSDTestBase(BaseTest):
    MAPPING_CLASS = "HL7_LCSD"
    DATA = ""


class LCSDTest1(LCSDTestBase):
    DATA = """MSH|^~\\&|SGL_EMETTEUR|ETABLISSEMENT_EMETTEUR&V|UNKNOWN|UNKNOWN|20100616060544||MFN^M10^MFN_M10|123456789|P|2.5|||||FRA|8859/15
MFI|OMC| ETABLISSEMENT_EMETTEUR_OMC_FRA_VERSION|REP||20120609000000|NE
MFE|MAD|||1|CE
OM1|1|DOC^11 DESOXYCORTICOSTERONE^L||Y| ETABLISSEMENT_EMETTEUR|||11 DESOXYCORTICOSTERONE||||||^LC/MS/MS^LN||^BIOLOGIE DE LA REPRODUCTION^NABM||A|||||1
OM5|1|DOC
ZCA|36&EUR|Y|N|N|||http://www.labtestsonline.fr/0150.html^^AP^HTML
OM4|1||||mL^Millitre^ISO+|PLAS^Plasma^ISO+||||REF^Réfrigéré^ISO+||1^mL"""

    def test_mapping(self):
        obj = self.F.parse(self.DATA)
        # Type HL7HD
        assert obj.emetteur.nsid.value == "ETABLISSEMENT_EMETTEUR"
        assert obj.emetteur.nsid.label == "V"
        assert obj.recepteur.nsid.value == "UNKNOWN"
        assert obj.recepteur_app.nsid.value == "UNKNOWN"
        # Type HL7MSG
        assert obj.type_message.message_type == "MFN"
        assert obj.type_message.event_type == "M10"
        assert obj.type_message.structure == "MFN_M10"
        assert obj.message_control_id == "123456789"
        assert obj.mode_traitement == "P"
        assert obj.version_id == "2.5"
        assert obj.encoding == "8859/15"
        assert obj.country_code == "FRA"


class LCSDTest2(LCSDTestBase):
    DATA = """MSH|^~\\&|SGL_EMETTEUR|ETABLISSEMENT_EMETTEUR|UNKNOWN|UNKNOWN|20100616060544||MFN^M10^MFN_M10|123456789|P|2.5|||||FRA|8859/15
MFI|OMC| ETABLISSEMENT_EMETTEUR_OMC_FRA_VERSION|REP||20120609000000|NE
MFE|MAD|||2|CE
OM1|1|Anti-ECT^AC ANTI-ANTIGENES NUCLEAIRES SOLUBLES Recherche^L||Y|ETABLISSEMENT_EMETTEUR|||AC ANTI-ANTIGENES NUCLEAIRES SOLUBLES Recherche~AC ANTI-ENA~AC ANTI-JO1~AC ANTI-RIBONUCLEOPROTEINES~AC ANTI-RNP~AC ANTI-Ro||||||FEIA^^L||^AUTO-IMMUNITE^NABM||A|||||1|||||||||||||||||Q5W|(SSA/SSB/SM/RNP/JO1/SCL70)\\S\\LORSQUE LA RECHERCHE EST POSITIVE, L'IDENTIFICATION EST REALISEE.
OM5|1|Anti-ECT
ZCA||Y|N|N||1456~1456| http://www.labtestsonline.fr/0150.html^^AP^HTML |
OM4|1|||| mL^Millitre^ISO+||SER^Serum^ISO+||||REF^Réfrigéré^ISO+||2^mL
MFE|MAD|||2|CE
OM1|2|Anti-ECT^AC ANTI-ANTIGENES NUCLEAIRES SOLUBLES Recherche^L||Y|ETABLISSEMENT_EMETTEUR|||AC ANTI-ANTIGENES NUCLEAIRES SOLUBLES Recherche~AC ANTI-ENA~AC ANTI-JO1~AC ANTI-RIBONUCLEOPROTEINES~AC ANTI-RNP~AC ANTI-Ro||||||FEIA^^L||^AUTO-IMMUNITE^NABM||A|||||1|||||||||||||||||Q5W|(SSA/SSB/SM/RNP/JO1/SCL70)\\S\\LORSQUE LA RECHERCHE EST POSITIVE, L'IDENTIFICATION EST REALISEE.
OM5|2|Anti-ECT
ZCA||Y|N|N||1456~1456| http://www.labtestsonline.fr/0150.html^^AP^HTML
OM4|2||||mL^Millitre^ISO+|PLAS^Plasma^ISO+||||REF^Réfrigéré^ISO+||2^mL|"""

    def test_mapping(self):
        obj = self.F.parse(self.DATA)
        # HprimMultiple
        assert obj.emetteur.nsid.value == "ETABLISSEMENT_EMETTEUR"


class LCSDTest3(LCSDTestBase):
    DATA = """MSH|^~\\&|SGL_EMETTEUR|ETABLISSEMENT_EMETTEUR|UNKNOWN|UNKNOWN|20100616060544||
MFI|OMC|ETABLISSEMENT_EMETTEUR_OMC_FRA_VERSION|REP||20110208094910|NE
MFE|MAD|||3|CE
OM1|1|477^ASPERGILLOSE Sérologie, dépistage (1/2ème dét.)^L||Y|ETABLISSEMENT_EMETTEUR||| ASPERGILLOSE Sérologie, dépistage (1/2ème dét.)~GALACTOMANNANE~DETECTION D'ANTIGENES~AC PRECIPITANT||||||ELS^^L||^MYCOLOGIE^NABM||A|||||1440|||||||||||||||||Q5W|
OM5|1|477|
ZCA||N|N|N||4307~4307~6307| http://www.labtestsonline.fr/0150.html^^AP^HTML|TARIFICATION VARIABLE SELON 1ERE ou 2EME DETERMINATION
OM4|1||||mL^Millitre^ISO+|SER^Sérum|||REF^Réfrigéré|1^mL"""

    def test_mapping(self):
        obj = self.F.parse(self.DATA)
        # HprimMultiple
        assert obj.emetteur.nsid.value == "ETABLISSEMENT_EMETTEUR"


class LCSDTest4(LCSDTestBase):
    DATA = """MSH|^~\\&|SGL_EMETTEUR|ETABLISSEMENT_EMETTEUR|UNKNOWN|UNKNOWN|20100616060544||MFN^M10^MFN_M10|123456789|P|2.5|||||FRA|8859/15
MFI|OMC|ETABLISSEMENT_EMETTEUR _OMC_FRA_VERSION|REP||20120609000000|NE
MFE|MAD|||2|CE
OM1|1|Anti-ECT^AC ANTI-ANTI^L||Y|ETABLISSEMENT_EMETTEUR|||AC ANTI-ANTI~AC ANTI-ENA~AC ANTI-JO1~AC ANTI-RIBONUCLEOPROTEINES~AC ANTI-RNP~AC ANTI-Ro||||||FEIA^^L||^AUTO-IMMUNITE^NABM||A|||||1|||||||||||||||||Q5W|(SSA/SSB/SM/RNP/JO1/SCL70)\\S\\LORSQUE LA RECHERCHE EST POSITIVE, L'IDENTIFICATION EST REALISEE.
OM5|1|Anti-ECT
ZCA||Y|N|N|N|1456~1456| http://www.labtestsonline.fr/0150.html^^AP^HTML |
OM4|1|||1|{Tube}^Tube^L|SER^Serum^ISO+|||REF^Réfrigéré^ISO+|2^{Tube}"""

    def test_mapping(self):
        obj = self.F.parse(self.DATA)
        # HprimMultiple
        assert obj.emetteur.nsid.value == "ETABLISSEMENT_EMETTEUR"


class LCSDTestUnicodeSplitters(LCSDTestBase):
    DATA = """MSH|^~\\&|OPENLAB|95.1&LABO_CERBA|UNKNOWN|UNKNOWN|20170904100004||MFN^M10^MFN_M10|20170904100004|P|2.5|||||FRA|8859/15|FRA
MFI|OMC|95.1&LABO_CERBA_OMC_FR_524|REP||20170904100004|NE
MFE|MAD|||1^95.1&LABO_CERBA|EI
OM1|1|1^11 DESOXYCORTICOSTERONE^L||Y|95.1^LABO_CERBA|||11 DESOXYCORTICOSTERONE~DOC||||||^LC-MS-MS^L||^BIOLOGIE DE LA REPRODUCTION^NABM||A|||||1440|||||||||||||||||1QW|Commentaire\x85avec unicode newlines
OM5|1|1|
ZCA|37.00&EUR|Y|N|N|||http://www.lab-cerba.com/pdf/0197F.pdf|
OM4|1||TUBE POUR PLASMA|||PLAS^PLASMA|||REF^REFRIGERE|1^mL"""

    def test_unicode_linesplit(self):
        msh = self.F.parse(self.DATA)
        assert (
            msh.mfi[0].mfe[0].om1[0].commentaire
            == "Commentaire\x85avec unicode newlines"
        )
